<?php declare(strict_types = 1);

namespace Drupal\comment_on_top\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Returns responses for Comment on top routes.
 */
final class CommentOnTopController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $messenger;

  /**
   * Constructs a new CommentOnTopController instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * Change all comments in this node from "1" to null in field stick on top.
   */
  private function changeAllLastCommentsToNull($node_id) {
    $comments = $this->entityTypeManager->getStorage('comment')->loadByProperties([
      'entity_id' => $node_id,
      'entity_type' => 'node',
    ]);

    foreach ($comments as $comment) {

      if ($comment->field_stick_on_top->stick === '1') {
        $comment->field_stick_on_top->setValue('0');
        $comment->save();
      }
    }
  }

  /**
   * Grant comment in this node from null to "1" in field stick on top.
   */
  public function stickOnTop($node_id, $comment_id) {
    $this->changeAllLastCommentsToNull($node_id);

    $comment = $this->entityTypeManager->getStorage("comment")->load($comment_id);
    $comment->field_stick_on_top->stick = '1';
    $comment->save();

    $this->messenger->addStatus($this->t('Comment is successfully sticked on top!'));
    $url = Url::fromRoute('entity.node.canonical', ['node' => $node_id]);
    return new RedirectResponse($url->toString());
  }

  /**
   * Remove comment in this node from "1" to null in field stick on top.
   */
  public function removeFromTop($node_id, $comment_id) {
    $comment = $this->entityTypeManager->getStorage("comment")->load($comment_id);
    $comment->field_stick_on_top->stick = '0';
    $comment->save();

    $this->messenger->addStatus($this->t('Comment is removed from top!'));
    $url = Url::fromRoute('entity.node.canonical', ['node' => $node_id]);
    return new RedirectResponse($url->toString());
  }
}
