<?php declare(strict_types = 1);

namespace Drupal\comment_on_top\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'stick_comment_on_top' field type.
 *
 * @FieldType(
 *   id = "stick_comment_on_top",
 *   label = @Translation("Stick on top"),
 *   category = @Translation("General"),
 *   default_widget = "stick_comment_on_top",
 *   default_formatter = "stick_comment_on_top_default",
 * )
 */
final class StickOnTopItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return $this->stick != 1;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {

    $properties['stick'] = DataDefinition::create('boolean')
      ->setLabel(t('Stick'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints(): array {
    $constraints = parent::getConstraints();

    // @todo Add more constraints here.
    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {

    $columns = [
      'stick' => [
        'type' => 'int',
        'size' => 'tiny',
      ],
    ];

    $schema = [
      'columns' => $columns,
      // @DCG Add indexes here if necessary.
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {

    $values['stick'] = (bool) mt_rand(0, 1);

    return $values;
  }

}
